;------------------------------------------------------------------------------
;
;        smloader
;                 a minimalist ZX Spectrum custom tape loader
;                                       by Sebastian Mihai, 2024
;                                       http://sebastianmihai.com
;
; As a kid, I was fascinated by my ZX Spectrum. One of its more mystical
; aspects was how certain games used custom loaders to load the bulk of their
; program data from tape.
;
; The default loader (found in the Sinclair-supplied ROM) is easily
; identifiable by the thin blue-yellow stripes which dance on the screen
; during loading.
;
; Custom loaders, on the other hand, replaced the default loader with very
; interesting effects: different border colours, countdown counters, etc.
;
; smloader is based on the Sinclair ZX Spectrum ROM tape loader routines,
; greatly reduced in size and functionality. User break, save, verify, etc.
; have all been stripped out. Aside from showing the bare minimum of a tape
; loader, smloader can also serve as a starting point for more interesting
; custom loaders.
;
; Loading a program has the following workflow:
; BASIC loader --> load smloader --> load program --> jump into program start
;   ^                   ^
;   |                   |
;   |                 this module
;   |
; generated automatically by pasmo assembler
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Values specific to the program-to-be-loaded
; (change these to use smloader for your own program)
;------------------------------------------------------------------------------
    LOAD_ADDRESS equ 40000  ; we will load the data block to here
                            ; obviously, this matches the ORG directive
                            ; declared by the data block's source code
                                    
    BLOCK_LENGTH equ 25281  ; data block's length; this matches the length
                            ; of the raw binary of the data block to load

;------------------------------------------------------------------------------
; By this point, the BASIC loader has executed, loading us (the custom loader)
; 
; We will make no assumptions about interrupt state and the presence of a 
; valid stack - thus setting them up ourselves
;------------------------------------------------------------------------------
main: org 32768             ; stay above ULA-contended memory
                            ; loading smloader below this may interfere with
                            ; timing, causing faults during tape loading
    
    di                      ; we disable interrupts to set up our stack, but
                            ; also because the loader main function requires it
                        
    ld sp, endstack         ; set up our new stack (we point to the end 
                            ; because the stack grows into lower addresses)

perform_load:
    ld ix, LOAD_ADDRESS     ; IX := destination address
    ld de, BLOCK_LENGTH     ; DE := block length
    call smloader_load_data ; CARRY := 1 on success
    jp c, success
    
error:
    jp 0                    ; reset computer
    
success:
    jp LOAD_ADDRESS         ; pass control to loaded block

    ;
    ; NOTE:
    ;           This is a bare-minimum stack size - large enough
    ;           for just this severely cut-down loader
    ;           If this loader is extended, the stack will most likely
    ;           have to be increased in size
    ;
    stack ds 6              ; reserve some bytes for our stack
    endstack:               ; mark end of our stack
    
    include "loader.asm"

end main                    ; needed by Pasmo assembler to generate
                            ; a BASIC loader
                        
